using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;

namespace derivedForm
{
    //  .
    class Ressore00
    {
        public Cargo00 c;

        PointF[] headers;   //      . 
        int N;              //   .
        PointF[] nodes;     //    (N+1).

        float h0;           //    . 
        float h1;           //    .

        public float kR;    //   .
                 

        //cylinderSize.Width;          //    .
        //cylinderSize.Height;         //    .
        SizeF cylinderSize;
        
        float area;                    //   .
 
        float step;                    //   .
        float rDirection;              // -1:1   .


        Ressore00(float x0, float y0, float h0, float h1, float step, float rDirection, float k)
        {
            headers = new PointF[] 
            {
             new PointF(x0,y0),
             new PointF(x0,y0)
            };

            this.h0 = h0;
            this.h1 = h1;

            this.step = step;
            this.rDirection = rDirection;
            this.kR = k;
        }

        //    .
        public float Step
        {
            set
            {
                step = value;            
            }        
        }

        //    .
        public float RD
        {
            set
            {
                rDirection = value;
            }
            get 
            {
                return rDirection;
            }
        }

        //   .
        public void ChangeRD()
        {
            rDirection *= -1;
        }

        public Ressore00(float x0, float y0, //   .               
                         float h0,           //    .
                         float h1,           //    . 
                         int n,              //   .
                         float width,        //   .
                         float height,       //   .
                         float step,
                         float rDirection,
                         float k
                         ) : this(x0, y0, h0, h1, step, rDirection, k)
        {
            int i, j, M = n+1;
            N = n;
            cylinderSize = new SizeF(width, height);
            area = width * height;

            float w = width / 2.0F; //    .
            float h = height / N;   //  .

            nodes = new PointF[N + 1];
            for (i = 0, j = -1, y0+=h0; i < M; i++, j *= -1)
            {
                nodes[i] = new PointF(x0+w*j, y0+=h);            
            }

            headers[1].Y = y0 + h1 + h;
        }
              
        //  . 
        public void rDown()
        {
            int i,j, M = N+1;

            float x0 = headers[0].X;
            float y0 = headers[0].Y;
            float h0 = this.h0;
            float h1 = this.h1;

            float hh = cylinderSize.Height;
            hh += step * rDirection;
            float ww = (float)(area/hh);
            cylinderSize = new SizeF(ww,hh);

            float w = (float)(ww/2.0F); //    .
            float h = (float)(hh/N);    //  .

            for (i = 0, j = -1, y0 += h0; i < M; i++, j *= -1)
            {
                nodes[i] = new PointF(x0 + (w*j), y0 += h);
            }

            headers[1].Y = y0 + h1 + h;

            c.Position = headers[1];        
        }

        
         //  .
        public void drawRessore(Graphics gr, Pen pen, double rPoint)
        {
            int i;
            for (i = 0; i < N; i++)
            {
                gr.DrawLine(
                            pen,
                            (float)(nodes[i].X * rPoint),
                            (float)(nodes[i].Y * rPoint),
                            (float)(nodes[i + 1].X * rPoint),
                            (float)(nodes[i + 1].Y * rPoint)
                            );
            }

            gr.DrawLine(
                        pen,
                        (float)(headers[0].X * rPoint),
                        (float)(headers[0].Y * rPoint),
                        (float)(headers[0].X * rPoint),
                        (float)((headers[0].Y + h0) * rPoint)
                        );


            gr.DrawLine(
                        pen,
                        (float)(nodes[0].X * rPoint),
                        (float)(nodes[0].Y * rPoint),
                        (float)(headers[0].X * rPoint),
                        (float)((headers[0].Y + h0) * rPoint)
                        );


            gr.DrawLine(
                        pen,
                        (float)(headers[1].X * rPoint),
                        (float)(headers[1].Y * rPoint),
                        (float)(headers[1].X * rPoint),
                        (float)((headers[1].Y - h1) * rPoint)
                        );

            gr.DrawLine(
                        pen,
                        (float)(nodes[i].X * rPoint),
                        (float)(nodes[i].Y * rPoint),
                        (float)(headers[1].X * rPoint),
                        (float)((headers[1].Y - h1) * rPoint)
                        );        
        }

    }
}
