using System;
using System.Drawing;
using System.Collections.Generic;
using System.Text;

namespace Win3dApp00
{
    class Matrix3d
    {
        static double toRad = Math.PI / 180.0;
        
        public const int REPLACE = 0;
        public const int PRECONCAT = 1;
        public const int POSTCONCAT = 2;

        float e00, e01, e02, e03;
        float e10, e11, e12, e13;
        float e20, e21, e22, e23;
        float e30, e31, e32, e33;

        public Matrix3d()
        {
            SetOne(); //  .
        }

        //     .
        public void SetOne()
        {
            e00 = 1.0F; e01 = 0.0F; e02 = 0.0F; e03 = 0.0F;
            e10 = 0.0F; e11 = 1.0F; e12 = 0.0F; e13 = 0.0F;
            e20 = 0.0F; e21 = 0.0F; e22 = 1.0F; e23 = 0.0F;
            e30 = 0.0F; e31 = 0.0F; e32 = 0.0F; e33 = 1.0F;
        }

                
        //  .
        public void Multiply(Matrix3d m)
        {
            Matrix3d A = this;
            Matrix3d B = m;

            float n00 = A.e00 * B.e00 + A.e10 * B.e01 + A.e20 * B.e02 + A.e30 * B.e03;
            float n01 = A.e01 * B.e00 + A.e11 * B.e01 + A.e21 * B.e02 + A.e31 * B.e03;
            float n02 = A.e02 * B.e00 + A.e12 * B.e01 + A.e22 * B.e02 + A.e32 * B.e03;
            float n03 = A.e03 * B.e00 + A.e13 * B.e01 + A.e23 * B.e02 + A.e33 * B.e03;

            float n10 = A.e00 * B.e10 + A.e10 * B.e11 + A.e20 * B.e12 + A.e30 * B.e13;
            float n11 = A.e01 * B.e10 + A.e11 * B.e11 + A.e21 * B.e12 + A.e31 * B.e13;
            float n12 = A.e02 * B.e10 + A.e12 * B.e11 + A.e22 * B.e12 + A.e32 * B.e13;
            float n13 = A.e03 * B.e10 + A.e13 * B.e11 + A.e23 * B.e12 + A.e33 * B.e13;

            float n20 = A.e00 * B.e20 + A.e10 * B.e21 + A.e20 * B.e22 + A.e30 * B.e23;
            float n21 = A.e01 * B.e20 + A.e11 * B.e21 + A.e21 * B.e22 + A.e31 * B.e23;
            float n22 = A.e02 * B.e20 + A.e12 * B.e21 + A.e22 * B.e22 + A.e32 * B.e23;
            float n23 = A.e03 * B.e20 + A.e13 * B.e21 + A.e23 * B.e22 + A.e33 * B.e23;

            float n30 = A.e00 * B.e30 + A.e10 * B.e31 + A.e20 * B.e32 + A.e30 * B.e33;
            float n31 = A.e01 * B.e30 + A.e11 * B.e31 + A.e21 * B.e32 + A.e31 * B.e33;
            float n32 = A.e02 * B.e30 + A.e12 * B.e31 + A.e22 * B.e32 + A.e32 * B.e33;
            float n33 = A.e03 * B.e30 + A.e13 * B.e31 + A.e23 * B.e32 + A.e33 * B.e33;

            e00 = n00;
            e01 = n01;
            e02 = n02;
            e03 = n03;

            e10 = n10;
            e11 = n11;
            e12 = n12;
            e13 = n13;

            e20 = n20;
            e21 = n21;
            e22 = n22;
            e23 = n23;
            
            e30 = n30;
            e31 = n31;
            e32 = n32;
            e33 = n33;

        }

        //  .
        //   "",   ,
        //    ""   .
        // 
        public void Translate(float x, float y, float z)
        {
            this.e03 += x;
            this.e13 += y;
            this.e23 += z;
        }

        //    ...
        public void Scale(float x, float y, float z)
        {
            this.e00 *= x;
            this.e11 *= y;
            this.e22 *= z;
        }

        //   ...
        public void Scale(float s)
        {
            this.e00 *= s;
            this.e11 *= s;
            this.e22 *= s;
        }
        
        //     X   theta...
        public void RotateX(Matrix3d m, double theta)
        {
            theta *= toRad;

            double theta_cos = Math.Cos(theta);
            double theta_sin = Math.Sin(theta);

            m.SetOne();
           
            m.e11 =  (float)theta_cos;
            m.e12 = -(float)theta_sin;
            m.e21 =  (float)theta_sin;
            m.e22 =  (float)theta_cos;

            this.Multiply(m);
        }

        //     Y   theta...
        public void RotateY(Matrix3d m, double theta)
        {
            theta *= toRad;

            double theta_cos = Math.Cos(theta);
            double theta_sin = Math.Sin(theta);

            m.SetOne();

            m.e00 =  (float)theta_cos;
            m.e02 =  (float)theta_sin;
            m.e20 = -(float)theta_sin;
            m.e22 =  (float)theta_cos;

            this.Multiply(m);
        }

        //     Z   theta...
        public void RotateZ(Matrix3d m, double theta)
        {
            theta *= toRad;

            double theta_cos = Math.Cos(theta);
            double theta_sin = Math.Sin(theta);

            m.SetOne();

            m.e00 = (float)theta_cos;
            m.e01 = -(float)theta_sin;
            m.e10 = (float)theta_sin;
            m.e11 = (float)theta_cos;

            this.Multiply(m);
        }
        
                
        //    3d  2d   .
        //    .
        public PointF Transformation2d(Point3d p)
        {
            float x = p.x * e00 + p.y * e01 + p.z * e02 + e03;
            float y = p.x * e10 + p.y * e11 + p.z * e12 + e13;
            float z = p.x * e20 + p.y * e21 + p.z * e22 + e23;

            
            return new PointF(x, y);
        }


        //    3d  2d   .
        //     .
        public PointF Transformation2d(Point3d p, float d)
        {
            float x = p.x * e00 + p.y * e01 + p.z * e02 + e03;
            float y = p.x * e10 + p.y * e11 + p.z * e12 + e13;
            float z = p.x * e20 + p.y * e21 + p.z * e22 + e23;

            float xx = (float)(d * x);
            float yy = (float)(d * y);

            return new PointF(xx, yy);
        }     
    }
}
